--[[
One possible clue is that in most if not all cases the people with the jacked up bars were out of range? I'm not 100% sure of that though.

]]--
local MAJOR, MINOR = "LibHermesUICooldownBars-1.0", 100001
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

-----------------------------------------------------------------------
-- LOCALS
-----------------------------------------------------------------------
local BARPOOL = nil
local BAR_SPARK_WIDTH = 10
local BAR_SPARK_EXTRA_HEIGHT = 10
local BAR_FONTGAP = 2
local UPDATE_FREQUENCY = 0.06 --just set it to match hermes frequency

-----------------------------------------------------------------------
-- SECONDARY DURATION TIMER
-----------------------------------------------------------------------
local SECANI_BREAKPOINT = 1 								--color change will start with this many seconds left

--GREEN to RED
--local SECANI_COLOR_LONG = {r = 0, g = 1, b = 0, a = 1 }
--local SECANI_COLOR_SHORT = {r = 1, g = 0, b = 0, a = 1 }

--RED to GREEN
local SECANI_COLOR_LONG = {r = 1, g = 0, b = 0, a = 1 }
local SECANI_COLOR_SHORT = {r = 0, g = 1, b = 0, a = 1 }

local SECANI_FADE = false										--set to true to animate the delta between SECANI_COLOR_LONG and SECANI_COLOR_SHORT over the duration of SECANI_BREAKPOINT
																--it looks fairly terrible depending on start and stop colors and the value of SECANI_BREAKPOINT

-----------------------------------------------------------------------
-- HELPERS
-----------------------------------------------------------------------
local function _findTableIndex(tbl, item)
	for index, i in ipairs(tbl) do
		if(i == item) then
			return index
		end
	end

	return nil
end

local function _deleteFromIndexedTable(tbl, item)
	local index = _findTableIndex(tbl, item) 
	if not index then error("failed to locate item in table") end
	tremove(tbl, index)
end

local function _secondsToClock(sSeconds)
	local nSeconds = tonumber(sSeconds)
	if not nSeconds then
		return nil
	end
	if nSeconds < 60 then
		--seconds
		return nil, nil, floor(nSeconds)
	else
		if(nSeconds > 3600) then
			--hours
			local nHours = floor(nSeconds / 3600)
			local nMins = floor(nSeconds / 60 - (nHours * 60))
			local nSecs = floor(nSeconds - nHours * 3600 - nMins * 60)
			--return ("%02.f:%02.f:%02.f"):format(nHours, nMins, nSecs)
			return nHours, nMins, nSecs
		else
			--minutes
			local nMins = floor(nSeconds / 60)
			local nSecs = floor(nSeconds - nMins * 60)
			--return ("%02.f:%02.f"):format(nMins, nSecs)
			return nil, nMins, nSecs
		end
	end
	
	--catch all to avoid any accidental nil errors
	return nil, nil, 0
end

local function _createBarPool()
	BARPOOL = CreateFrame("Frame", nil, UIParent)
	BARPOOL:SetPoint("CENTER")
	BARPOOL:SetWidth(50)
	BARPOOL:SetHeight(50)
	BARPOOL:Hide()

	BARPOOL:EnableMouse(false)
	BARPOOL:SetMovable(false)
	BARPOOL:SetToplevel(false)
	BARPOOL.Bars = {}
end

local function _createBar()
	--bar frame
	local bar = CreateFrame("Frame", nil, nil)
	bar:Hide() -- hide by default
	bar:ClearAllPoints()
	bar:EnableMouse(false) --allow clickthrough
	bar:SetMovable(false)
	bar:SetResizable(false)
	bar:SetToplevel(true)
	
	--bar defaults
	bar.width = 100
	bar.height = 20
	bar.iconside = "left"
	bar.nameside = "left"
	bar.textratio = 50
	bar.animationdir = "right"
	bar.animationstyle = "empty"
	bar.fganimating = nil
	bar.lastonupdate = nil
	bar.secanimating = nil
	
	bar.bg = bar:CreateTexture(nil, "BACKGROUND")
	bar.bg:ClearAllPoints()
	bar.bg:Hide()
	
	bar.fg = bar:CreateTexture(nil, "BORDER")
	bar.fg:ClearAllPoints()

	--secondary animation texture
	bar.sectexture = bar:CreateTexture(nil, "BORDER")
	bar.sectexture:ClearAllPoints()
	bar.sectexture:Hide()
	bar.secspark = bar:CreateTexture(nil, "OVERLAY")
	bar.secspark:ClearAllPoints()
	bar.secspark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	bar.secspark:SetWidth(BAR_SPARK_WIDTH)
	bar.secspark:SetBlendMode("ADD")
	bar.secspark:Hide()
	
	bar.icon = bar:CreateTexture(nil, "OVERLAY")
	bar.icon:ClearAllPoints()
	bar.icon:SetTexture("")
	bar.icon:SetTexCoord(0.1,0.9,0.1,0.9)
	
	--bar ltext
	bar.text = bar:CreateFontString(nil, "ARTWORK", "GameFontWhite")
	bar.text:ClearAllPoints()
	bar.text:SetWordWrap(false)
	bar.text:SetJustifyV("CENTER") --defaults, could change
	bar.text:SetJustifyH("LEFT") --defaults, could change
	bar.text:SetText("")
	bar.text:SetTextColor(1, 0, 0, 0)
	bar.text:SetShadowColor(0, 0, 0, 0.5) --default include drop shadow
	bar.text:SetShadowOffset(1, -1) --default include drop shadow
	
	--bar rtext
	bar.duration = bar:CreateFontString(nil, "ARTWORK", "GameFontWhite")
	bar.duration:ClearAllPoints()
	bar.duration:SetWordWrap(false)
	bar.duration:SetJustifyV("CENTER") --defaults, could change
	bar.duration:SetJustifyH("RIGHT") --defaults, could change
	bar.duration:SetText("")
	bar.duration:SetTextColor(1, 0, 0, 0)
	bar.duration:SetShadowColor(0, 0, 0, 0.5) --default include drop shadow
	bar.duration:SetShadowOffset(1, -1) --default include drop shadow

	--spark for internal cooldowns
	bar.spark = bar:CreateTexture(nil, "OVERLAY")
	bar.spark:ClearAllPoints()
	bar.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	bar.spark:SetWidth(BAR_SPARK_WIDTH)
	bar.spark:SetBlendMode("ADD")
	bar.spark:Hide()
	
	lib:SetForegroundTexture(bar, "")
	lib:SetForegroundTextureColor(bar, 0, 0, 1, 1)
	
	lib:SetBackgroundTexture(bar, "")
	lib:SetBackgroundTextureColor(bar, 0, 0, 0, 1)
	
	lib:SetSecondaryTextureColor(bar, SECANI_COLOR_LONG.r, SECANI_COLOR_LONG.g, SECANI_COLOR_LONG.b, SECANI_COLOR_LONG.a)
	
	return bar
end

local function _getTextWidths(bar)
	local maxw = bar.bg:GetWidth() --always utilize the width of the bg texture for this, not the width of the bar because it's different if the icon is showing
	local wt, wd
	
	--account for 0%/100% ratios
	if bar.textratio <= 0 then
		wt = 0
		wd = maxw
	elseif bar.textratio >= 100 then
		wt = maxw
		wd = 0
	else
		wt = maxw * (bar.textratio / 100)
		wd = maxw - wt
	end
	
	return wt, wd
end

do
	_createBarPool()
end

function lib:AcquireBar()
	local bar
	if(#BARPOOL.Bars > 0) then
		bar = BARPOOL.Bars[1]
		_deleteFromIndexedTable(BARPOOL.Bars, bar)
	else
		bar = _createBar()
	end

	return bar
end

function lib:ReleaseBar(bar)
	tinsert(BARPOOL.Bars, bar)
	
	lib:StopForegroundAnimation(bar)
	
	bar:Hide()
	bar:ClearAllPoints()
	bar:SetParent(BARPOOL)
	
	--bar defaults
	bar.width = 100
	bar.height = 20
	bar.iconside = "left"
	bar.nameside = "left"
	bar.textratio = 50
	bar.animationdir = "right"
	bar.animationstyle = "empty"
	bar.fganimating = nil
	bar.lastonupdate = nil
	bar.secanimating = nil
	
	self:SetBackgroundTexture(bar, "")
	self:SetBackgroundTextureColor(bar, 0, 0, 1, 1)
	bar.bg:ClearAllPoints()
	bar.bg:Hide()
	
	self:SetForegroundTexture(bar, "")
	self:SetForegroundTextureColor(bar, 1, 0, 1, 1)
	bar.fg:ClearAllPoints()
	
	--secondary animation
	bar.sectexture:Hide()
	self:SetSecondaryTextureColor(bar, SECANI_COLOR_LONG.r, SECANI_COLOR_LONG.g, SECANI_COLOR_LONG.b, SECANI_COLOR_LONG.a)
	bar.sectexture:ClearAllPoints()
	bar.secspark:Hide()
	bar.secspark:ClearAllPoints()
	
	self:SetIconTexture(bar, "")
	bar.icon:ClearAllPoints()
	
	bar.text:SetFormattedText("")
	bar.text:SetFontObject("GameFontWhite")
	bar.text:ClearAllPoints()
	
	bar.duration:SetFormattedText("")
	bar.duration:SetFontObject("GameFontWhite")
	bar.duration:ClearAllPoints()
	
	bar.spark:Hide()
	bar.spark:ClearAllPoints()
	bar.spark:SetVertexColor(0, 1, 1, 1)
end

function lib:SetBackgroundTexture(bar, texture)
	bar.bg:SetTexture(texture)
end

function lib:SetBackgroundTextureColor(bar, r, g, b, a)
	bar.bg:SetVertexColor(r, g, b, a)
end

function lib:SetForegroundTexture(bar, texture)
	bar.fg:SetTexture(texture)
	bar.sectexture:SetTexture(texture)
end

function lib:SetForegroundTextureColor(bar, r, g, b, a)
	bar.fg:SetVertexColor(r, g, b, a)
	bar.spark:SetVertexColor(r, g, b, a)
end

function lib:SetSecondaryTextureColor(bar, r, g, b, a)
	bar.sectexture:SetVertexColor(r, g, b, a)
	bar.secspark:SetVertexColor(r, g, b, a)
end

function lib:SetFont(bar, filename, height, usedropshadow)
	bar.text:SetFont(filename, height)
	bar.duration:SetFont(filename, height)
	
	if usedropshadow == true then
		bar.text:SetShadowColor(0, 0, 0, 0.5)
		bar.text:SetShadowOffset(1, -1)
		bar.duration:SetShadowColor(0, 0, 0, 0.5)
		bar.duration:SetShadowOffset(1, -1)
	else
		bar.text:SetShadowColor(0, 0, 0, 0)
		bar.text:SetShadowOffset(0, 0)
		bar.duration:SetShadowColor(0, 0, 0, 0)
		bar.duration:SetShadowOffset(0, 0)
	end
end

function lib:SetFontColorText(bar, r, g, b, a)
	bar.text:SetTextColor(r, g, b, a)
end

function lib:SetFontColorDuration(bar, r, g, b, a)
	bar.duration:SetTextColor(r, g, b, a)
end

function lib:SetIconTexture(bar, texture)
	bar.icon:SetTexture(texture)
end

function lib:SetWidth(bar, width, redraw)
	bar.width = width
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetHeight(bar, height, redraw)
	bar.height = height
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetIconLocation(bar, iconside, redraw)
	bar.iconside = iconside
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetNameSide(bar, nameside, redraw)
	bar.nameside = nameside
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetTextRatio(bar, textratio, redraw)
	bar.textratio = textratio
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetAnimationDirection(bar, animationdir, redraw)
	bar.animationdir = animationdir
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetAnimationStyle(bar, animationstyle, redraw)
	bar.animationstyle = animationstyle
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir, bar.animationstyle)
	end
end

function lib:SetSize(bar, width, height, redraw)
	bar.width = width
	bar.height = height
	if redraw then
		self:Draw(bar, bar.width, bar.height, bar.iconside, bar.nameside, bar.textratio, bar.animationdir)
	end
end

function lib:Draw(bar, width, height, iconside, nameside, textratio, animationdir, animationstyle)
	if nameside ~= "right" then nameside = "left" end							--text on left by default
	if textratio == nil then textratio = 50 end									--50:50 ratio by default
	if animationdir ~= "left" then animationdir = "right" end					--animate to the right by default
	if animationstyle ~= "full" then animationstyle = "empty" end					--default style is empty
	
	bar.width = width
	bar.height = height
	bar.iconside = iconside
	bar.nameside = nameside
	bar.textratio = textratio
	bar.animationdir = animationdir
	bar.animationstyle = animationstyle
	
	bar.bg:ClearAllPoints()
	bar.fg:ClearAllPoints()
	bar.icon:ClearAllPoints()
	bar.text:ClearAllPoints()
	bar.duration:ClearAllPoints()
	bar.spark:ClearAllPoints()
	
	bar:SetWidth(bar.width)
	bar:SetHeight(bar.height)
	
	---------------------------
	--layout icon
	---------------------------
	if iconside == "right" then
		bar.icon:SetPoint("TOPRIGHT", bar, "TOPRIGHT", 0, 0)
		bar.icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", 0, 0)
		bar.icon:Show()
	elseif iconside == "none" then
		bar.icon:Hide()
	else
		--default is left
		bar.icon:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
		bar.icon:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
		bar.icon:Show()
	end
	
	bar.icon:SetWidth(bar.height)
	
	---------------------------
	--layout textures
	---------------------------
	if iconside == "right" then
		--set bg texture
		bar.bg:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
		bar.bg:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
		bar.bg:SetWidth(bar.width - bar.height)
		
		--set fg texture, depends on animationdir, do not set anchor points on both left and right sides
		if animationdir == "right" then
			--flows from left to right ---->
			bar.fg:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
			bar.fg:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		else
			--flows from right to left <----
			bar.fg:SetPoint("TOPRIGHT", bar.icon, "TOPLEFT", 0, 0)
			bar.fg:SetPoint("BOTTOMRIGHT", bar.icon, "BOTTOMLEFT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPRIGHT", bar.icon, "TOPLEFT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMRIGHT", bar.icon, "BOTTOMLEFT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		end
	elseif iconside == "none" then
		--set bg texture
		bar.bg:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
		bar.bg:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
		bar.bg:SetWidth(bar.width)
		
		--set fg texture, depends on animationdir, do not set anchor points on both left and right sides
		if animationdir == "right" then
			--flows from left to right ---->
			bar.fg:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
			bar.fg:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		else
			--flows from right to left <----
			bar.fg:SetPoint("TOPRIGHT", bar, "TOPRIGHT", 0, 0)
			bar.fg:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPRIGHT", bar, "TOPRIGHT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		end
	else --default is left
		--set bg texture
		bar.bg:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT", 0, 0)
		bar.bg:SetPoint("BOTTOMLEFT", bar.icon, "BOTTOMRIGHT", 0, 0)
		bar.bg:SetWidth(bar.width - bar.height)
		
		--set fg texture, depends on animationdir, do not set anchor points on both left and right sides
		if animationdir == "right" then
			--flows from left to right ---->
			bar.fg:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT", 0, 0)
			bar.fg:SetPoint("BOTTOMLEFT", bar.icon, "BOTTOMRIGHT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMLEFT", bar.icon, "BOTTOMRIGHT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		else
			--flows from right to left <----
			bar.fg:SetPoint("TOPRIGHT", bar, "TOPRIGHT", 0, 0)
			bar.fg:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", 0, 0)
			if not bar.fganimating then bar.fg:SetWidth(bar.bg:GetWidth()) end
			
			bar.sectexture:SetPoint("TOPRIGHT", bar, "TOPRIGHT", 0, 0)
			bar.sectexture:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", 0, 0)
			if not bar.secanimating then bar.sectexture:SetWidth(bar.bg:GetWidth()) end
		end
	end
	
	---------------------------
	--layout fonts
	---------------------------
	local wt, wd = _getTextWidths(bar)
	if wt > 0 then bar.text:SetWidth(wt) end
	if wd > 0 then bar.duration:SetWidth(wd) end
	
	if iconside == "right" then
		if nameside == "left" then
			bar.text:SetPoint("TOPLEFT", bar, "TOPLEFT", BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", BAR_FONTGAP, 0)
			bar.text:SetJustifyH("LEFT")
			
			bar.duration:SetPoint("TOPRIGHT", bar.icon, "TOPLEFT", -BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMRIGHT", bar.icon, "BOTTOMLEFT", -BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("RIGHT");
		else
			bar.text:SetPoint("TOPRIGHT", bar.icon, "TOPLEFT", -BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMRIGHT", bar.icon, "BOTTOMLEFT", -BAR_FONTGAP, 0)
			bar.text:SetJustifyH("RIGHT");
			
			bar.duration:SetPoint("TOPLEFT", bar, "TOPLEFT", BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("LEFT")
		end
	elseif iconside == "none" then
		if nameside == "left" then
			bar.text:SetPoint("TOPLEFT", bar, "TOPLEFT", BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", BAR_FONTGAP, 0)
			bar.text:SetJustifyH("LEFT")
			
			bar.duration:SetPoint("TOPRIGHT", bar, "TOPRIGHT", -BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", -BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("RIGHT");
		else
			bar.text:SetPoint("TOPRIGHT", bar, "TOPRIGHT", -BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", -BAR_FONTGAP, 0)
			bar.text:SetJustifyH("RIGHT");
			
			bar.duration:SetPoint("TOPLEFT", bar, "TOPLEFT", BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("LEFT")
		end
	else
		--default is left
		if nameside == "left" then
			bar.text:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT", BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMLEFT", bar.icon, "BOTTOMRIGHT", BAR_FONTGAP, 0)
			bar.text:SetJustifyH("LEFT")
			
			bar.duration:SetPoint("TOPRIGHT", bar, "TOPRIGHT", -BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", -BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("RIGHT");
		else
			bar.text:SetPoint("TOPRIGHT", bar, "TOPRIGHT", -BAR_FONTGAP, 0)
			bar.text:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", -BAR_FONTGAP, 0)
			bar.text:SetJustifyH("RIGHT");
			
			bar.duration:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT", BAR_FONTGAP, 0)
			bar.duration:SetPoint("BOTTOMLEFT", bar.icon, "BOTTOMRIGHT", BAR_FONTGAP, 0)
			bar.duration:SetJustifyH("LEFT")
		end
	end	

	---------------------------
	--layout spark
	---------------------------
	if animationdir == "right" then
		bar.spark:SetPoint("CENTER", bar.fg, "RIGHT", 0, 0)
		bar.secspark:SetPoint("CENTER", bar.sectexture, "RIGHT", 0, 0)
	else
		bar.spark:SetPoint("CENTER", bar.fg, "LEFT", 0, 0)
		bar.secspark:SetPoint("CENTER", bar.sectexture, "LEFT", 0, 0)
	end
	
	bar.spark:SetHeight(bar.height + BAR_SPARK_EXTRA_HEIGHT)
	bar.secspark:SetHeight(bar.height + BAR_SPARK_EXTRA_HEIGHT)
end

function lib:AnimateForegroundTexture(bar, initial, current)
	--flag that the foreground animation has started
	bar.fganimating = 1
	
	--we want to set the size of the texture even if the secondary animation is running, because the secondary animation will eventally show
	local maxw = bar.bg:GetWidth() --always utilize the width of the bg texture for this, not the width of the bar because it's different if the icon is showing
	if bar.animationstyle == "full" then --starts out full
		width = (current / initial) * maxw
		--just some simple error handling to prevent crazy math errors or bad calls to animate
		if width >= maxw then
			width = maxw
		end
		bar.fg:SetWidth(width)
	else  --starts out empty
		width = maxw - ((current / initial) * maxw)
		--if current / initial == 0 then we need to manually set the size to 1. Anything other than zero!! Otherwise the bar size doesn't actuall change
		if width <= 1 then
			width = 1
		end
		bar.fg:SetWidth(width)
	end

	--don't set the text or do anything with spark and background if the secondary animation is running
	if not bar.secanimating then
		local h, m, s = _secondsToClock(current)
		if h then
			bar.duration:SetFormattedText("%d:%02.f:%02.f", h, m, s)
		elseif m then
			bar.duration:SetFormattedText("%d:%02.f", m, s)
		else
			bar.duration:SetFormattedText("%d", s)
		end
		
		bar.spark:Show()
		bar.bg:Show()
	end
end

function lib:StopForegroundAnimation(bar)
	bar.fganimating = nil
	bar.spark:Hide()
	bar.fg:SetWidth(bar.bg:GetWidth())
	
	--this is just a safe check to ensure that any style changes are reset. The assumption being made here is that it's never proper to have a secondary animation >= the foreground animation
	self:StopSecondaryAnimation(bar)
	
	--only do the following if the secondary animation isn't running
	bar.duration:SetFormattedText("")
	bar.bg:Hide()
end

--------------------------------------------------------------------
-- SECONDARY ANIMATION
--------------------------------------------------------------------
local function _calculateFade(sr, sg, sb, sa, er, eg, eb, ea, duration)
	local inc = duration / SECANI_BREAKPOINT
	local dr, dg, db, da = er + ((sr - er) * inc), eg + ((sg - eg) * inc), eb + ((sb - eb) * inc), ea + ((sa - ea) * inc)
	return dr, dg, db, da
end

local function OnBarSecondaryAnimation(bar, elapsed)
	bar.lastonupdate = bar.lastonupdate + elapsed
	bar.secremaining = bar.secremaining - elapsed --update the remainder on the secondary animation
	while (bar.lastonupdate > UPDATE_FREQUENCY) do
		bar.lastonupdate = bar.lastonupdate - UPDATE_FREQUENCY
		
		if bar.secremaining <= 0 then
			--stop the animation if it's completed, this will kill the event and reset everything correctly
			lib:StopSecondaryAnimation(bar)
			return
		end
		
		if bar.secremaining <= SECANI_BREAKPOINT then
			if SECANI_FADE then
				local r, g, b, a = _calculateFade(
					SECANI_COLOR_LONG.r,
					SECANI_COLOR_LONG.g,
					SECANI_COLOR_LONG.b,
					SECANI_COLOR_LONG.a,
					SECANI_COLOR_SHORT.r,
					SECANI_COLOR_SHORT.g,
					SECANI_COLOR_SHORT.b,
					SECANI_COLOR_SHORT.a,
					bar.secremaining)
				lib:SetSecondaryTextureColor(bar, r, g, b, a)
			else
				lib:SetSecondaryTextureColor(bar, SECANI_COLOR_SHORT.r, SECANI_COLOR_SHORT.g, SECANI_COLOR_SHORT.b, SECANI_COLOR_SHORT.a)
			end
		end
	
		local maxw = bar.bg:GetWidth() --always utilize the width of the bg texture for this, not the width of the bar because it's different if the icon is showing
		local h, m, s = _secondsToClock(bar.secremaining)
		if h then
			bar.duration:SetFormattedText("%d:%02.f:%02.f", h, m, s)
		elseif m then
			bar.duration:SetFormattedText("%d:%02.f", m, s)
		else
			bar.duration:SetFormattedText("%d", s)
		end
		
		if bar.animationstyle == "full" then --starts out full
			width = (bar.secremaining / bar.secinitial) * maxw
			--just some simple error handling to prevent crazy math errors or bad calls to animate
			if width >= maxw then
				width = maxw
			end
			bar.sectexture:SetWidth(width)
		else  --starts out empty
			width = maxw - ((bar.secremaining / bar.secinitial) * maxw)
			--if bar.secremaining / bar.secinitial == 0 then we need to manually set the size to 1. Anything other than zero!! Otherwise the bar size doesn't actuall change
			if width <= 1 then
				width = 1
			end
			bar.sectexture:SetWidth(width)
		end
		bar.sectexture:Show()
		bar.secspark:Show()
		bar.bg:Show()
	end
end

function lib:StartSecondaryAnimation(bar, duration) --start, stop, and initial are all assumed to be positive values in seconds
	--just do some quick error handling because ultimately the data is untrusted
	local number = tonumber(duration)
	if number == nil or number <= 0 or number > 3600 then return end --assume nothing would EVER go higher than 60 minutes!
	
	bar.secinitial = number
	bar.secremaining = number
	bar.lastonupdate = 0
	bar.secanimating = 1
	
	--hide the fg animation or textures regardless of if they're running
	bar.fg:Hide()
	bar.spark:Hide()
	
	if number <= SECANI_BREAKPOINT then
		--figure out the initial color to use
		lib:SetSecondaryTextureColor(bar, SECANI_COLOR_SHORT.r, SECANI_COLOR_SHORT.g, SECANI_COLOR_SHORT.b, SECANI_COLOR_SHORT.a)
	else
		lib:SetSecondaryTextureColor(bar, SECANI_COLOR_LONG.r, SECANI_COLOR_LONG.g, SECANI_COLOR_LONG.b, SECANI_COLOR_LONG.a)
	end
	
	bar:SetScript("OnUpdate", OnBarSecondaryAnimation)
end

function lib:StopSecondaryAnimation(bar)
	bar.lastonupdate = nil
	bar:SetScript("OnUpdate", nil)
	bar.secanimating = nil
	
	bar.duration:SetFormattedText("") --empty out the duration, if fg animation is running it'll get put back to a proper value
	bar.sectexture:Hide()
	bar.secspark:Hide()
		
	--only restore the following if the foreground animation is still running
	if bar.fganimating then
		bar.fg:Show()
		bar.spark:Show()
	end
end




